/**
 * Hlavní semestrální práce z KIV/VSP
 * (c) 2006 - 2007 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.sem.application;

import cz.habi.vsp.sem.distribution.ExponentialDistribution;
import cz.habi.vsp.sem.network.CrossingTarget;
import cz.habi.vsp.sem.network.FlowCrossing;
import cz.habi.vsp.sem.network.NetworkExit;
import cz.habi.vsp.sem.network.RequestGenerator;
import cz.habi.vsp.sem.network.Server;
import cz.habi.vsp.sem.network.SystemSimulation;

/**
 * @author habi
 *
 */
/*
 * Jednoduchy priklad na sit front. Lepsi nez popis je mozna maly ASCII obrazek:
 * 
 * 
 * 
 * 
 *  Start1-->---->-->--Server1---X>--->---Konec
 *               |               v
 *               |               |
 *               ---<--Server2-<-< 
 *                               | 
 *                               |
 *  Start2-->---------------------
 *
 * 
 * Hlavni program tedy "pouze" vytvori instance prislusnych objektu
 * a spravne je "pospojuje". Pak se spusti simulace, ktera bezi pro
 * nejaky pocet pozadavku
 * 
 * PS: Cele je to pouze priklad:-)
 *
 */
public class ExampleSimulation {
	
	/* parametry simulace */
	private static final double lambdaStart1 = 0.5;

	private static final double lambdaStart2 = 0.2;

	private static final double lambdaServer1 = 2;

	private static final double lambdaServer2 = 1.5;

	// pravdepodobnost, ze to z Crossu pujde smerem do EndPlace
	private static final double ppstGoEnd = 0.8;

	// kolik nechat projit simulaci pozadavku
	private static final int LINK_COUNT = 100000;
	
	public static void main(String[] args) {
		Tools.supressJSimLogging();
		
		try {
			SystemSimulation simulation = new SystemSimulation("Sim", LINK_COUNT);
			
			Server server1 = new Server("S1", simulation, new ExponentialDistribution(lambdaServer1));
			Server server2 = new Server("S2", simulation, new ExponentialDistribution(lambdaServer2));
			
			RequestGenerator rg1 = new RequestGenerator("G1", simulation, new ExponentialDistribution(lambdaStart1));
			RequestGenerator rg2 = new RequestGenerator("G2", simulation, new ExponentialDistribution(lambdaStart2));
			
			NetworkExit exit = new NetworkExit(simulation);
			
			FlowCrossing fc = new FlowCrossing(2);
			CrossingTarget ctEnd = new CrossingTarget(ppstGoEnd, exit);
			CrossingTarget ctBack = new CrossingTarget(1 - ppstGoEnd, server2);
			fc.addDispatcherTarget(ctEnd);
			fc.addDispatcherTarget(ctBack);
			
			rg1.setNextRequestAcceptor(server1);
			server1.setNextRequestAcceptor(fc);
			rg2.setNextRequestAcceptor(server2);
			server2.setNextRequestAcceptor(server1);
			
			server1.activateNow();
			server2.activateNow();
			rg1.activateNow();
			rg2.activateNow();
			
			simulation.registerStatisticsProvider(server1);
			simulation.registerStatisticsProvider(server2);
			
			simulation.registerStatisticsProvider(server1.getQueueStatistics());
			simulation.registerStatisticsProvider(server2.getQueueStatistics());

			simulation.runSimulation();

			Tools.printStatisticsLaTeX(System.out, simulation.getStatisticProviders());
			
//			Tools.printStatistics(System.out, simulation);
			
//			System.out.println("Steps: " + simulation.getStepsCount());


			
//			for (StatisticsProvider sp: simulation.getStatisticProviders()) {
//				Tools.printStatistics(System.out, sp);
//			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
}
