/**
 * Hlavní semestrální práce z KIV/VSP
 * (c) 2006 - 2007 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.sem.application;

import java.io.PrintStream;
import java.util.Collection;

import cz.habi.vsp.sem.network.StatisticsProvider;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * @author habi
 * 
 */
public final class Tools {

	private Tools() {
		// empty
	}

	/**
	 * Sets the JSim logging level to WARNING.
	 */
	public static void supressJSimLogging() {
		// switching off the default JSim logging
		java.util.logging.Logger l1 = java.util.logging.Logger.getLogger(JSimProcess.class.getName());
		l1.setLevel(java.util.logging.Level.WARNING);

		java.util.logging.Logger l2 = java.util.logging.Logger.getLogger(JSimSimulation.class.getName());
		l2.setLevel(java.util.logging.Level.WARNING);
	}

	/**
	 * Prints statistics to the print stream
	 * 
	 * @param out
	 *            print stream
	 * @param sp
	 *            statiscics provider
	 */
	public static void printStatistics(PrintStream out, Collection<StatisticsProvider> stats) {
		for (StatisticsProvider sp : stats) {
			out.println("+=========================================");
			out.println("| name:  " + sp.getProviderName());
			out.println("| class: " + sp.getClass().getName());
			for (String s : sp.getStatistics().keySet()) {
				out.println("|   " + s + ": " + sp.getStatistics().get(s));
			}
			out.println("+-----------------------------------------");
		}
	}
	
	public static void printStatisticsTabs(PrintStream out, Collection<StatisticsProvider> stats) {
		for (StatisticsProvider sp : stats) {
			out.println("name\t" + sp.getProviderName());
			out.println("class\t" + sp.getClass().getName());
			for (String s : sp.getStatistics().keySet()) {
				out.println(s + "\t" + sp.getStatistics().get(s));
			}
		}
	}

	/**
	 * Prints the statistics in LaTeX format.
	 * 
	 * @param out
	 *            output print stream
	 * @param stats
	 *            collection of statistics
	 * @throws IllegalArgumentException
	 *             if out is null or stats are null or empty
	 */
	public static void printStatisticsLaTeX(PrintStream out, Collection<StatisticsProvider> stats) throws IllegalArgumentException {
		if (out == null) {
			throw new IllegalArgumentException("out is null");
		}

		if (stats == null || stats.size() == 0) {
			throw new IllegalArgumentException("stats are null or empty");
		}

		out.println("\\begin{description}");

		for (StatisticsProvider sp : stats) {
			out.println("\\item[" + sp.getProviderName() + "] (\\emph{class:} \\verb=" + sp.getClass().getName() + "= )");
			out.println("\\begin{center}");
			out.println("\\begin{tabular}{|l|l|} \\hline");
			out.println("Veličina & Hodnota \\\\ \\hline");
			for (String s : sp.getStatistics().keySet()) {
				out.println("\\verb=" + s + "= & " + sp.getStatistics().get(s) + "\\\\ \\hline");
			}
			out.println("\\end{tabular}");
			out.println("\\end{center}");
		}

		out.println("\\end{description}");
	}
}
