/**
 * Hlavní semestrální práce z KIV/VSP
 * (c) 2006 - 2007 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.sem.network;

/**
 * This class represents a single request which is passed through the quiuing
 * network.
 * 
 * @author habi
 * 
 */
public class Request {

	// time, when the request was created
	private double creatingTime;

	/**
	 * Creates a new instance of Request
	 * 
	 * @param creatingTime
	 *            creating time
	 */
	public Request(double creatingTime) {
		this.creatingTime = creatingTime;
	}

	/**
	 * Returns the value of creatingTime
	 * 
	 * @return time
	 */
	public double getCreatingTime() {
		return creatingTime;
	}

	private double enteringQueueTime;

	private boolean waitingInQueue = false;

	private Double waitingTimeInLastQueue = null;

	/**
	 * This method should be called always when the request enters the queue.
	 * 
	 * @param enteringQueueTime
	 *            time, when the request arrives in the queue
	 * @throws IllegalStateException
	 *             if the request is already in a queue
	 */
	public void enterQueue(double enteringQueueTime) throws IllegalStateException {
		if (this.waitingInQueue) {
			throw new IllegalStateException("The request is still in the queue!");
		}

		this.enteringQueueTime = enteringQueueTime;
		this.waitingInQueue = true;
	}

	/**
	 * This method shloud be called, when the request leaves the queue.
	 * 
	 * @param leavingQueueTime
	 *            time of leaving the queue
	 * @throws IllegalStateException
	 *             if the request is not in queue
	 */
	public void leaveQueue(double leavingQueueTime) throws IllegalStateException {
		if (!waitingInQueue) {
			throw new IllegalStateException("The request is not in any queue!");
		}

		waitingTimeInLastQueue = leavingQueueTime - enteringQueueTime;
		this.waitingInQueue = false;
	}

	/**
	 * Returns the time, that the request spent (was waiting) in the last
	 * "visited" queue.
	 * 
	 * @return time of waiting or null, if the request has not yet been in any
	 *         queue
	 */
	public Double getWaitingTimeInLastQueue() {
		return waitingTimeInLastQueue;
	}

}
