/**
 * Hlavní semestrální práce z KIV/VSP
 * (c) 2006 - 2007 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.sem.network;

import cz.habi.vsp.sem.distribution.Distribution;
import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimProcess;

/**
 * This class generates requests. It's lifecycle ends when all reqired request
 * have been generated.
 * 
 * @author habi
 * 
 */
public class RequestGenerator extends NetworkProcess {
	private Integer numbersCount = null;

	private int numbersAlreadyGenerated = 0;

	/**
	 * Creates a new instance of {@link RequestGenerator} with required
	 * parameters.
	 * 
	 * @param name
	 *            name of process
	 * @param simulation
	 *            parent simulation
	 * @param distribution
	 *            probability distribution
	 * @param numbersCount
	 *            Count of numbers to be generated. If null, the generator works
	 *            forever.
	 * @throws JSimException
	 *             if an error occurs while creating {@link JSimProcess}
	 * @throws IllegalArgumentException
	 *             if the numbersCount is not-null and negative
	 */
	public RequestGenerator(String name, SystemSimulation simulation, Distribution distribution, Integer numbersCount)
			throws JSimException, IllegalArgumentException {
		super(name, simulation, distribution);

		if (numbersCount != null) {
			if (numbersCount <= 0) {
				throw new IllegalArgumentException("Count of numbers must greater than zero!");
			} else {
				this.numbersCount = numbersCount;
			}
		}
	}

	/**
	 * Creates a new instance of {@link RequestGenerator} with required
	 * parameters.
	 * 
	 * @param name
	 *            name of process
	 * @param simulation
	 *            parent simulation
	 * @param distribution
	 *            probability distribution
	 * @throws JSimException
	 *             if an error occurs while creating {@link JSimProcess}
	 * @throws IllegalArgumentException
	 *             if the numbersCount is not-null and negative
	 */
	public RequestGenerator(String name, SystemSimulation simulation, Distribution distribution) throws JSimException,
			IllegalArgumentException {
		this(name, simulation, distribution, null);
	}

	@Override
	protected void doWork() throws JSimException {
		// generate a new request
		Request request = new Request(myParent.getCurrentTime());
		numbersAlreadyGenerated++;

		// update system statistics
		((SystemSimulation) getParent()).requestEnteredToSystem();

		// deliver it to the next requestAcceptor
		getNextRequestAcceptor().acceptRequest(request);

		// and wait some time for the next request
		holdForRandomTime();
	}

	@Override
	protected boolean workingCondition() {
		if (numbersCount != null) {
			return (numbersAlreadyGenerated <= numbersCount);
		}

		return true;
	}
}
